/* 
    kwintv using Bt848 frame grabber driver

    Copyright (C) 1998 Moritz Wenk (wenk@mathematik.uni-kl.de)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

// Hey, better do not change the order of the includes !!!
// REMEMBER: KDE & Qt-stuff BEFORE X11-stuff !!!

#include <wintv.h>

#include <unistd.h>

#include <qtooltip.h>
#include <qfiledlg.h>
#include <qkeycode.h>
#include <qmsgbox.h>

#include <wintvmain.h>

#include <kiconloader.h>
#include <klocale.h>
#include <kwm.h>

#include <countries.h>

//#undef HAVE_LIBXXF86VM
#ifdef HAVE_LIBXXF86VM
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/xf86vmode.h>
#include <X11/extensions/xf86vmstr.h>
XF86VidModeModeInfo **vm_modelines;
int vm_count;
#endif

#ifdef X11DEBUG
#include "xeventtype.h"
#endif

#define BID_UP      0
#define BID_DOWN    1
#define BID_CAP     2 
#define BID_SS      3
#define BID_FAR     4
#define BID_FSS     5
#define BID_VUP     6
#define BID_MUTE    7
#define BID_VDOWN   8
#define BID_SC      9
#define BID_EXIT   10
#define BID_LCD    11
#define BID_VSL    12

#define SB_PROG_TEXT  1
#define SB_PROG       2
#define SB_SNAP_TEXT  3
#define SB_SNAP       4 
#define SB_VOL_TXT    5
#define SB_VOL        6

#define ID_HIDETOOL   0
#define ID_HIDESTATUS 1
#define ID_HIDEALL    2
#define ID_HIDEMOUSE  3

#define COMMIT     // commit actions like channel saving etc.

#define KEY10UP Key_AsciiCircum

winTVMainWidget::winTVMainWidget( int argc, char **argv, QWidget* parent,
				  const char* name, rcfile *_rc )
  :  KTopLevelWidget( name ), rc(_rc)
{
  int i;

  lcd=0;
  winTVScreen=NULL;

#ifdef mainDEBUG
  debug("main: begin of init");
#endif

  KWM::setIcon(winId(), kapp->getIcon());
  KWM::setMiniIcon(winId(), kapp->getMiniIcon());

  setCaption((const char*)(QString("kWinTV ")+QString(VERSION)));
  // set icon
  
  KIconLoader *loader = kapp->getIconLoader();
#ifdef mainDEBUG
  debug("main: %s",(const char*)kapp->kde_icondir());
#endif

  pm = loader->loadIcon("kwintv_xwintv.xpm");
  setIcon(pm);
  setIconText("kWinTV");

  QStrList *rlist= NULL;
#ifdef HAVE_LIBXXF86VM
  Display *dpy= qt_xdisplay();
  int  foo,bar,ma,mi;
  char dummy[255];
  rlist= new QStrList();

  if (XF86VidModeQueryExtension(dpy,&foo,&bar)) {
    XF86VidModeQueryVersion(dpy,&ma,&mi);
    if (ma == 0 && mi < 8) {
      warning("main: VidMode server extention v0.8 or newer required, disabled");
      have_vm= false;
    } else if ((ma != XF86VIDMODE_MAJOR_VERSION) || (mi != XF86VIDMODE_MINOR_VERSION)) {
      warning("main: VidMode server extention version mismatch, disabled");
      warning("main: server version %d.%d != included version %d.%d",ma,mi,
	      XF86VIDMODE_MAJOR_VERSION,XF86VIDMODE_MINOR_VERSION); 
      have_vm= false;
    } else {
      have_vm = true;
      XF86VidModeGetAllModeLines(dpy,XDefaultScreen(dpy),&vm_count,&vm_modelines);
#ifdef mainDEBUG
      debug("main: X-Server supports VidMode extention (version %d.%d)",ma,mi); 
      debug("main:  available video mode(s):");
      for (i = 0; i < vm_count; i++)
	debug("main: %dx%d",vm_modelines[i]->hdisplay,vm_modelines[i]->vdisplay);
#endif
      for (i = 0; i < vm_count; i++) {
	sprintf(dummy,"%d x %d",vm_modelines[i]->hdisplay,vm_modelines[i]->vdisplay);
	rlist->append(dummy);
      }
    }
  }
#else
  warning("main: ----- Video Mode extention disabled -----"); 
#endif

  // user keys
  initKKeyConfig(kapp->getConfig()); 
  userKeys= KKeyConfig::getKKeyConfig(); 
  userKeys->registerWidget("wintvmain",this);
  userKeys->addKey("program up",Key_Up);
  userKeys->connectFunction("wintvmain","program up",this,SLOT(channelDownSLOT()));
  userKeys->addKey("program down",Key_Down);
  userKeys->connectFunction("wintvmain","program down",this,SLOT(channelUpSLOT()));
  userKeys->addKey("volume up",Key_Right);
  userKeys->connectFunction("wintvmain","volume up",this,SLOT(volumeUpSLOT()));
  userKeys->addKey("volume down",Key_Left);
  userKeys->connectFunction("wintvmain","volume down",this,SLOT(volumeDownSLOT()));
  userKeys->addKey("stereo mixer toggle",CTRL+Key_S);
  userKeys->connectFunction("wintvmain","stereo mixer toggle",this,SLOT(splitSlidersExtSLOT()));
  userKeys->addKey("add new program",CTRL+Key_A);
  userKeys->connectFunction("wintvmain","add new program",this,SLOT(addChannelSLOT()));
  userKeys->addKey("edit active program",CTRL+Key_E);
  userKeys->connectFunction("wintvmain","edit active program",this,SLOT(editChannelSLOT()));
  userKeys->addKey("delete active program",CTRL+Key_D);
  userKeys->connectFunction("wintvmain","delete active program",this,SLOT(deleteChannelSLOT()));
  userKeys->addKey("video clip",Key_V);
  userKeys->connectFunction("wintvmain","video clip",this,SLOT(videoClipSLOT()));
  userKeys->addKey("capture toggle",Key_C);
  userKeys->connectFunction("wintvmain","capture toggle",this,SLOT(captureToggleSLOT()));
  userKeys->addKey("snap shot",Key_S);
  userKeys->connectFunction("wintvmain","snap shot",this,SLOT(screenShotSLOT()));
  userKeys->addKey("mute toggle",CTRL+Key_M);
  userKeys->connectFunction("wintvmain","mute toggle",this,SLOT(soundSLOT()));
  userKeys->addKey("fix aspect ratio",Key_A);
  userKeys->connectFunction("wintvmain","fix aspect ratio",this,SLOT(adjustAspectRatioSLOT()));
  userKeys->addKey("full screen toggle",Key_F);
  userKeys->connectFunction("wintvmain","full screen toggle",this,SLOT(fullScreenSLOT()));
  userKeys->addKey("full size toggle",Key_D);
  userKeys->connectFunction("wintvmain","full size toggle",this,SLOT(fullSizeSLOT()));
  userKeys->addKey("toggle mouse cursor",Key_M);
  userKeys->connectFunction("wintvmain","toggle mouse cursor",this,SLOT(mousecursorToggleSLOT()));
  userKeys->addKey("general setup",ALT+Key_G);
  userKeys->connectFunction("wintvmain","general setup",this,SLOT(setupSLOT()));
  userKeys->addKey("audio auto",ALT+Key_A);
  userKeys->connectFunction("wintvmain","audio auto",this,SLOT(audioAutoSLOT()));
  userKeys->addKey("audio mono",ALT+Key_M);
  userKeys->connectFunction("wintvmain","audio mono",this,SLOT(audioMonoSLOT()));
  userKeys->addKey("audio stereo",ALT+Key_S);
  userKeys->connectFunction("wintvmain","audio stereo",this,SLOT(audioStereoSLOT()));
  userKeys->addKey("audio language 1",ALT+Key_1);
  userKeys->connectFunction("wintvmain","audio language 1",this,SLOT(audioL1SLOT()));
  userKeys->addKey("audio language 2",ALT+Key_2);
  userKeys->connectFunction("wintvmain","audio language 2",this,SLOT(audioL2SLOT()));
  userKeys->addKey("key bindings",ALT+Key_K);
  userKeys->connectFunction("wintvmain","key bindings",this,SLOT(userKeysConfigSLOT()));
  userKeys->addKey("toggle program box",Key_B);
  userKeys->connectFunction("wintvmain","toggle program box",this,SLOT(toggleChannelBoxSLOT()));
  userKeys->addKey("exit kwintv",Key_Q);
  userKeys->connectFunction("wintvmain","exit kwintv",kapp,SLOT(quit()));
  userKeys->addKey("toggle Menus",CTRL-Key_T);
  userKeys->connectFunction("wintvmain","toggle Menus",this,SLOT(toggleTVonlySLOT()));
  //userKeys->addKey("",Key_);
  //userKeys->connectFunction("wintvmain","",this,SLOT());

  // irc stuff
#ifdef HAVE_IRC
  ircCodes= new KIRCWidget( kapp->getConfig() );
  ircCodes->registerWidget("wintvmain",this);
  ircCodes->addCode("program up",0xb1);
  ircCodes->connectFunction("wintvmain","program up",this,SLOT(channelDownSLOT()));
  ircCodes->addCode("program down",0xb0);
  ircCodes->connectFunction("wintvmain","program down",this,SLOT(channelUpSLOT()));
  ircCodes->addCode("volume up",146);
  ircCodes->connectFunction("wintvmain","volume up",this,SLOT(volumeUpSLOT()));
  ircCodes->addCode("volume down",147);
  ircCodes->connectFunction("wintvmain","volume down",this,SLOT(volumeDownSLOT()));
  ircCodes->addCode("snap shot",0xb8);
  ircCodes->connectFunction("wintvmain","snap shot",this,SLOT(screenShotSLOT()));
  ircCodes->addCode("mute toggle",188);
  ircCodes->connectFunction("wintvmain","mute toggle",this,SLOT(soundSLOT()));
  ircCodes->addCode("full screen toggle",0xce);
  ircCodes->connectFunction("wintvmain","full screen toggle",this,SLOT(fullScreenSLOT()));
  ircCodes->addCode("toggle mouse cursor",0x9d);
  ircCodes->connectFunction("wintvmain","toggle mouse cursor",this,SLOT(mousecursorToggleSLOT()));
  ircCodes->addCode("capture toggle",0xb2);
  ircCodes->connectFunction("wintvmain","capture toggle",this,SLOT(captureToggleSLOT()));

  ircCodes->addCode("0",0xa0, KCodeEntry::number );
  ircCodes->addCode("1",0x80, KCodeEntry::number );
  ircCodes->addCode("2",0x81, KCodeEntry::number );
  ircCodes->addCode("3",0x82, KCodeEntry::number );
  ircCodes->addCode("4",0x83, KCodeEntry::number );
  ircCodes->addCode("5",0x84, KCodeEntry::number );
  ircCodes->addCode("6",0x85, KCodeEntry::number );
  ircCodes->addCode("7",0x86, KCodeEntry::number );
  ircCodes->addCode("8",0x87, KCodeEntry::number );
  ircCodes->addCode("9",0x88, KCodeEntry::number );
  ircCodes->addCode("+10",0xa1,KCodeEntry::up10 );
  ircCodes->addCode("+20",0xa2,KCodeEntry::up20 );
  ircCodes->addCode("+30",0xa3,KCodeEntry::up30 );
  ircCodes->addCode(">10",0xa7,KCodeEntry::twoDigits );

  connect( ircCodes, SIGNAL(numKeyReceivedSIGNAL(int)), 
	   this, SLOT(switchToChannelSLOT(int)));
  connect( ircCodes, SIGNAL(specialKeyReceivedSIGNAL( KCodeEntry::codeType )), 
	   this, SLOT( displaySpecialKeySLOT( KCodeEntry::codeType )));
#endif // HAVE_IRC

  // load icons
  ch_up_xpm     = loader->loadIcon("kwintv_ch_up.xpm");
  ch_down_xpm   = loader->loadIcon("kwintv_ch_down.xpm");
  capture_xpm   = loader->loadIcon("kwintv_capture.xpm");
  uncapture_xpm = loader->loadIcon("kwintv_capture.xpm");
  snap_xpm      = loader->loadIcon("kwintv_snap.xpm");
  aspect_xpm    = loader->loadIcon("kwintv_aspect.xpm");
  fullsize_xpm  = loader->loadIcon("kwintv_fullsize.xpm");
  vol_up_xpm    = loader->loadIcon("kwintv_vol_up.xpm");
  mute_xpm      = loader->loadIcon("kwintv_mute.xpm");
  vol_down_xpm  = loader->loadIcon("kwintv_vol_down.xpm");
  cs_xpm        = loader->loadIcon("kwintv_cs.xpm");
  exit_xpm      = loader->loadIcon("kwintv_exit.xpm");
  
  // toolbar
  toolbar = new KToolBar(this,"kwintv -- ToolBar --",26);
  toolbar->setBarPos(KToolBar::Top);
  toolbar->insertButton(ch_down_xpm,BID_UP,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Switch program down"));

  // channel number lcd
  lcd = new QLCDNumber(toolbar, "LCD" );
  lcd->setMinimumSize( 1, 1 );
  lcd->setMaximumSize( 40, 40 );
  {
    QColorGroup normal( QColor( QRgb(9355867) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );
    QColorGroup disabled( QColor( QRgb(8421504) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(8421504) ), QColor( QRgb(12632256) ) );
    QColorGroup active( QColor( QRgb(0) ), QColor( QRgb(12632256) ), QColor( QRgb(16777215) ), QColor( QRgb(6316128) ), QColor( QRgb(10789024) ), QColor( QRgb(0) ), QColor( QRgb(16777215) ) );
    QPalette palette( normal, disabled, active );
    lcd->setPalette( palette );
  }
  lcd->setSmallDecimalPoint( false );
  lcd->setNumDigits( 2 );
  lcd->setMode( QLCDNumber::DEC );
  lcd->setSegmentStyle( QLCDNumber::Flat );
  lcd->setFrameStyle( QFrame::NoFrame );

  toolbar->insertWidget(BID_LCD,26,lcd);
  
  toolbar->insertButton(ch_up_xpm,BID_DOWN,SIGNAL(released(int)),this,
			 SLOT(buttonsSLOT(int)),true,i18n("Switch program up"));
  toolbar->insertSeparator();

  toolbar->insertButton(uncapture_xpm,BID_CAP,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Toggle capture"));
  toolbar->setToggle(BID_CAP);
  if ( rc->capture ) toolbar->toggleButton(BID_CAP);

  toolbar->insertButton(snap_xpm,BID_SS,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Snapshot"));
  toolbar->insertSeparator();

  toolbar->insertButton(aspect_xpm,BID_FAR,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Fix aspect ratio"));
  toolbar->insertButton(fullsize_xpm,BID_FSS,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Toggle full screen size"));
  toolbar->insertSeparator();

#if 0
  volslider= new QSlider( this, "Slider" );
  volslider->setOrientation( QSlider::Horizontal );
  volslider->setMinimumSize( 10, 10 );
  volslider->setMaximumSize( 100, 100 );
  volslider->setRange( 0, MIXER_MAX_VALUE );
  volslider->setSteps( 1, 10 );
  volslider->setValue( MIXER_MAX_VALUE );
  volslider->setTracking( TRUE );
  volslider->setTickmarks( QSlider::NoMarks );
  volslider->setTickInterval( 5 );

  toolbar->insertWidget(BID_VSL,80,volslider); 

  toolbar->insertButton(mute_xpm,BID_MUTE,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Mute/Unmute sound"));
  toolbar->setToggle(BID_MUTE);
  if ( !rc->sound_on ) toolbar->toggleButton(BID_MUTE);

#else
  toolbar->insertButton(vol_up_xpm,BID_VUP,SIGNAL(released(int)),this,
  			SLOT(buttonsSLOT(int)),true,i18n("Volume up"));
  toolbar->insertButton(mute_xpm,BID_MUTE,SIGNAL(toggled(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Mute/Unmute sound"));
  toolbar->setToggle(BID_MUTE);
  toolbar->insertButton(vol_down_xpm,BID_VDOWN,SIGNAL(released(int)),this,
  		SLOT(buttonsSLOT(int)),true,i18n("Volume down"));
#endif
  toolbar->insertSeparator();

  toolbar->insertButton(cs_xpm,BID_SC,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Show/Hide program box"));
  toolbar->insertButton(exit_xpm,BID_EXIT,SIGNAL(released(int)),this,
			SLOT(buttonsSLOT(int)),true,i18n("Exit"));

  addToolBar(toolbar);

 

  // popups for menubar  
  QPopupMenu *impmenu=new QPopupMenu;
  impmenu->insertItem("x&awtv",this,SLOT(importXAWTVChannelsSLOT()));
  impmenu->insertItem("&xtvscreen",this,SLOT(importXTVSCREENChannelsSLOT()));

  // options menu
  optmenu=new QPopupMenu;
  optmenu->setCheckable( true );
  omid[ID_HIDETOOL]=optmenu->insertItem(i18n("Hide Toolbar"),this,SLOT(toggleToolBarSLOT()));
  optmenu->setItemChecked(omid[ID_HIDETOOL],false);
  omid[ID_HIDESTATUS]=optmenu->insertItem(i18n("Hide Statusbar"),this,SLOT(toggleStatusBarSLOT()));
  optmenu->setItemChecked(omid[ID_HIDESTATUS],false);
  omid[ID_HIDEALL]=optmenu->insertItem(i18n("Hide All"),this,SLOT(toggleTVonlySLOT()));
  optmenu->setItemChecked(omid[ID_HIDEALL],false);
  omid[ID_HIDEMOUSE]=optmenu->insertItem(i18n("Hide Mousecursor"),this,SLOT(mousecursorToggleSLOT()));
  optmenu->setItemChecked(omid[ID_HIDEMOUSE],false);
  optmenu->insertSeparator();
  optmenu->insertItem(i18n("General &Options"),this,SLOT(setupSLOT()));
  optmenu->insertItem(i18n("&Key Bindings"),this,SLOT(userKeysConfigSLOT()));
#ifdef HAVE_IRC
  optmenu->insertItem(i18n("&IRC Code Bindings"),this,SLOT(ircCodesConfigSLOT()));
#endif
  optmenu->insertSeparator();
  optmenu->insertItem(i18n("&Save Options"),this,SLOT(saveRCSLOT()));

  // file menu
  QPopupMenu *filemenu=new QPopupMenu;
  filemenu->insertItem(i18n("&New"),this,SLOT(newChannelsSLOT()));
  filemenu->insertItem(i18n("&Load ..."),this,SLOT(loadChannelsSLOT()));
  filemenu->insertItem(i18n("&Save "),this,SLOT(saveChannelsSLOT()));
  filemenu->insertItem(i18n("Save &As ..."),this,SLOT(saveChannelsAsSLOT()));
  filemenu->insertSeparator();
  filemenu->insertItem(i18n("&Import"),impmenu);
  filemenu->insertSeparator();
  filemenu->insertItem(i18n("E&xit"),kapp,SLOT(quit()));

  // channel menu	
  chmenu=new QPopupMenu;
  chmenu->insertItem(i18n("&Show/Hide Program Box"),this,SLOT(toggleChannelBoxSLOT()));
  chmenu->insertSeparator();
  chmenu->insertItem(i18n("&Add"),this,SLOT(addChannelSLOT()));
  chmenu->insertItem(i18n("&Edit"),this,SLOT(editChannelSLOT()));
  chmenu->insertItem(i18n("&Cut"),this,SLOT(deleteChannelSLOT()));
  chmenu->insertItem(i18n("&Copy"),this,SLOT(copyChannelSLOT()));
  chmenu->insertItem(i18n("&Paste"),this,SLOT(pasteChannelSLOT()));
  chmenu->insertSeparator();
  chmenu->insertItem(i18n("&Scan for Programs"),this,SLOT(scanChannelsSLOT()));

  // audio menu
  //int audio=winTVScreen->getAudioMode();
  int audio=1;
  smenu=new QPopupMenu;
  smenu->setCheckable( true );
  smid[0]=smenu->insertItem(i18n("&Autodetection"),this,SLOT(audioAutoSLOT()));
  smenu->setItemChecked(smid[0], audio == 0); 
  smid[1]=smenu->insertItem(i18n("&Mono"),this,SLOT(audioMonoSLOT()));
  smenu->setItemChecked(smid[1], audio == VIDEO_SOUND_MONO); 
  smid[2]=smenu->insertItem(i18n("&Stereo"),this,SLOT(audioStereoSLOT()));
  smenu->setItemChecked(smid[2], audio == VIDEO_SOUND_STEREO); 
  smid[3]=smenu->insertItem(i18n("Language &1"),this,SLOT(audioL1SLOT()));
  smenu->setItemChecked(smid[3], audio == 3); 
  smid[4]=smenu->insertItem( i18n("Language &2"),this,SLOT(audioL2SLOT()));
  smenu->setItemChecked(smid[4], audio == 4); 
  smenu->insertSeparator();
  smid[5]=smenu->insertItem(i18n("M&ute Sound"),this,SLOT(soundSLOT()));
  smenu->setItemChecked(smid[5], !rc->sound_on); 
  smenu->insertSeparator();
  smid[6]=smenu->insertItem(i18n("S&tereo Mixer"),this,SLOT(splitSlidersSLOT()));
  smenu->setItemChecked(smid[6], !rc->sliders_att_startup ); 

  // television menu
  tvmenu=new QPopupMenu;
  tvmenu->setCheckable( true );
  tvmid[0]=tvmenu->insertItem(i18n("&Capture on"),this,SLOT(captureToggleSLOT()));
  tvmenu->setItemChecked(tvmid[0],rc->capture);
  tvmid[1]=tvmenu->insertItem(i18n("&Full Screen"),this,SLOT(fullScreenSLOT()));
  tvmenu->setItemChecked(tvmid[1],false);
  tvmid[2]=tvmenu->insertItem(i18n("F&ull Size"),this,SLOT(fullSizeSLOT()));
  tvmenu->setItemChecked(tvmid[2],false);
  tvmenu->insertItem(i18n("Fix &Aspect Ratio"),this,SLOT(adjustAspectRatioSLOT()));
  tvmenu->insertSeparator();
  tvmenu->insertItem(i18n("&Program"),chmenu);
  tvmenu->insertItem(i18n("A&udio"),smenu);
  tvmenu->insertSeparator();
  tvmenu->insertItem(i18n("&Snap Shot"),this,SLOT(screenShotSLOT()));
  tvmenu->insertItem(i18n("&Video Clip"),this,SLOT(videoClipSLOT()));

  // help menu
  QString about(i18n(
	     QString("                 kWinTV version ")+QString(VERSION)+QString("\n\n") 
	    +QString("            kWinTV is a KDE application for\n"
		     "           Video4Linux frame grabber TV cards.\n\n"
		     "         Copyright 1998, 1999 by Moritz Wenk\n\n"
		     "This Program is distributed under the terms of the\n"
		     "             GNU General Public License\n\n"
		     "Send any comments, gifts, homages, abusings etc. to:\n" 
		     "              wenk@mathematik.uni-kl.de\n\n"
		     "          For new versions, have a look at:\n"
		     "  http://www.mathematik.uni-kl.de/~wenk/xwintv.html")));

  QPopupMenu *helpmenu=KApplication::getKApplication()->getHelpMenu(true,about); 
  helpmenu->insertSeparator();
  helpmenu->insertItem(i18n("&Properties..."),this,SLOT(propertiesSLOT()));

  // menu bar  
  menubar = new KMenuBar( this, "kwintv -- MenuBar --" );
  menubar->insertItem(i18n("&File"),filemenu);
  menubar->insertItem(i18n("&TeleVision"),tvmenu);
#ifdef HAVE_VTX
  // we dont have vtx till now...
  //menubar->insertItem(i18n("&VideoText"),this,SLOT(vtxSLOT()));
#endif
  menubar->insertItem(i18n("&Options"),optmenu);
  menubar->insertSeparator();
  menubar->insertItem(i18n("&Help"),helpmenu);
  setMenu(menubar);

  /////////////////////////////////////////////////////////////////////////////
  statusbar= new KStatusBar(this,"statusbar");
  statusbar->insertItem("Program:",SB_PROG_TEXT);
  statusbar->insertItem("WWWWWWWWWWWW",SB_PROG);
  statusbar->changeItem((const char*)rc->channelListName,SB_PROG);
  statusbar->insertItem("Snap Shot:",SB_SNAP_TEXT);
  statusbar->insertItem("0000",SB_SNAP);
  statusbar->insertItem("Volume:",SB_VOL_TXT);
  statusbar->insertItem("00%",SB_VOL);
  
  setStatusBar(statusbar);

  ///////////////////////////////////////////////////////////////////////
  // load channel-file
  //channel_list.setAutoDelete ( true );
  channel_list.setAutoDelete ( false );
  loadChannels((const char*)(rc->dataDir+QString("/")+rc->channelListName));

  /////////////////////////////////////////////////////////////////////////////
  // init sound mixer
  mix= new mixer((const char *)rc->mixerDeviceName,(unsigned int)rc->mixerDevice);
  rc->mixerDevice=(int)mix->getDevice();
  //  mix->set_vol(0,0);

  /////////////////////////////////////////////////////////////////////////////
  // resize etc.  
  if ( rc->ol_restore ) {
#ifdef mainDEBUG
    debug("main: ol_restore");
#endif
    setGeometry(rc->ol_geometry);
  } else {
    resize(400,300+toolbar->height()+menubar->height()+statusbar->height() );
  }
  setMinimumSize( 384/2,288/2 );

  /////////////////////////////////////////////////////////////////////////
  // set up winTVScreen widget
  char kv4lsetup[128] = "kv4lsetup -q";

  // first parse some command line options
  int bpp=0;
  int dual=0;
  for ( i=0; i < argc; i++ ) {
    if ( strcmp(argv[i],"-bpp") == 0 ) { 
      if ( argc >= i+1 ) {
	i++;
	bpp=atoi(argv[i]);
	switch (bpp) {
	case 8:
	case 15:
	case 16:
	case 24:
	case 32:
	  strcat(kv4lsetup," -b ");
	  strcat(kv4lsetup,argv[i]);
	  break;
	default:
	  fatal(i18n("main: incorrect value %d given for switch -bpp. Valid values are 8,15,16,24,32."),bpp);
	}
      } else fatal(i18n("main: no parameter given for switch -bpp!"));
    }
    if ( (strcmp(argv[i],"-dev") == 0) || (strcmp(argv[i],"-device") == 0) ) {
      if ( argc >= i+1 ) {
	i++; 
	rc->bttvDeviceName=argv[i];
      }
    }
    if ( (strcmp(argv[i],"-cap") == 0) || (strcmp(argv[i],"-capture") == 0) ) {
      rc->capture=1;
    }
    if ( (strcmp(argv[i],"-dual") == 0) ) {
      dual=1;
    }
    if ( (strcmp(argv[i],"-x") == 0) ) {
      have_vm=false;
    }
    if ( (strcmp(argv[i],"-h") == 0) || (strcmp(argv[i],"-help") == 0) ) {
      printf(i18n("This is kwintv " 
      "\n Aviable command line switches: \n"
      "\t -device DEVICENAME         Set bttv device to DEVICENAME (default is /dev/bttv).\n"
      "\t -bpp [8|15|16|24|32]       Force colour depth of given value.\n"
      "\t -capture                   Set capture on after startup. Default is off. USE WITH CARE!\n"
      "\t -dual                      Force dual mode for graphic cards with dual buffers.\n"
      "\t -x                         Disable use of VidModeExtension.\n"
      "See README for details.\n\n"));
      kapp->quit();
      exit(0);
    }
  }

  fixed=true;
#if 1
  strcat(kv4lsetup," -c ");
  strcat(kv4lsetup,rc->bttvDeviceName);
  switch (system(kv4lsetup)) {
  case -1:
    warning("main: could'nt start kv4lsetup!\n");
    break;
  case 0:
#ifdef mainDEBUG
  debug("main: kv4lsetup well done...");
#endif
    break;
  default:
    warning("main: kv4lsetup had some trouble, trying to continue anyway.\n");
    break;
  }
#endif
  // now set up ...

  winTVScreen = new winTVScreenWidget(this,
				      "winTVScreen",
				      rc->capture,
				      (const char *) rc->bttvDeviceName,
				      bpp,dual);
	
  // set winTVScreen values
  if (rc->screensaver) winTVScreen->toggleScreenSaverSLOT();

  winTVScreen->setTVPopup(tvmenu);
  winTVScreen->showChannelInfoEnableSLOT( 0 );
  setView(winTVScreen);

  // does tuner see STEREO ???
  // FIX ME
  /*
  audio=winTVScreen->getAudioMode();
  if ( audio == VIDEO_SOUND_MONO ) {
  smenu->setItemEnabled (smid[2],false);
  smenu->setItemEnabled (smid[3],false);
  smenu->setItemEnabled (smid[4],false);
  }
  */

  /////////////////////////////////////////////////////////////////////////////
  // set up ctab dialogs
#ifdef mainDEBUG
  debug("main: init of ctab");
#endif
  tab= new ctab(NULL,"ctab");
  options= new coptions(tab,"options");
  selector= new cselector(tab,"selector",rc,
			  winTVScreen->getCapCap(),
			  winTVScreen->getCapChannel(),
			  winTVScreen->getCapTuner(),
			  winTVScreen->getCapAudio(),
			  winTVScreen);
  //  freq= new cfreq(tab,"freq");
  // cfreq for later version

  tab->addTab(selector,"&Channel");
  tab->addTab(options,"&Options");
  //tab->addTab(freq,"&Frequency");

  tab->setCancelButton();
  tab->setOkButton();
  tab->setDefaultButton();

  /////////////////////////////////////////////////////////////////////////////
  // set up otab dialogs
#ifdef mainDEBUG
  debug("main: init of otab");
#endif
  mtab= new otab(NULL,"otab");
  general= new ogeneral(mtab,"gen");
  mtab->addTab(general,"&General");
  mixdev= new omixdev(mtab,"mixdev",mix,winTVScreen->getCanVolume());
  mtab->addTab(mixdev,"&Mixer");
  aspectratio= new oaspectratio(mtab,"ar");
  mtab->addTab(aspectratio,"&Aspect Ratio");
  snapshot= new osnapshot(mtab,"ss");
  mtab->addTab(snapshot,"&Snap Shot");
  //#ifdef HAVE_LIBXXF86VM
  fullscreen= new ofullscreen(mtab,"fs",rlist);
  mtab->addTab(fullscreen,"&Full Screen");
  //#endif

  mtab->setOkButton();
  mtab->setCancelButton();
  mtab->setDefaultButton();

  /////////////////////////////////////////////////////////////////////////
  // connect everything
  connect(winTVScreen,SIGNAL(keyPressedSIGNAL(int)),this,SLOT(keyPressedSLOT(int)));
  connect(winTVScreen,SIGNAL(autoDetectAudioModeSIGNAL(int)),this,SLOT(audioModeChangedSLOT(int)));
  connect(winTVScreen,SIGNAL(wroteVideoClipFrameSIGNAL(int)),this,SLOT(frameVClipSLOT(int)));
  connect(winTVScreen,SIGNAL(lostVideoClipFrameSIGNAL(int)),this,SLOT(framelostVClipSLOT(int)));
  connect(winTVScreen,SIGNAL(statVideoClipFrameSIGNAL(double)),this,SLOT(statVideoClipFrameSLOT(double)));

  connect(winTVScreen,SIGNAL(toggleTVonlySIGNAL()),this,SLOT(toggleTVonlySLOT()));
  connect(this,SIGNAL(channelChangedSIGNAL()),this,SLOT(channelChangedSLOT()));

  // connect capture changing
  connect(this,SIGNAL(captureToggleSIGNAL(int)),winTVScreen,SLOT(changeCaptureSLOT(int)));

  // connect audio
  connect(this,SIGNAL(audioModeChangedSIGNAL(int)),winTVScreen,SLOT(changeAudioModeSLOT(int)));
  connect(this,SIGNAL(audioMuteToggledSIGNAL()),winTVScreen,SLOT(toggleAudioMuteSLOT()));
  // FIX ME FIX ME
  //if (rc->use_tv_volume) 
  connect(this,SIGNAL(volumeChangedSIGNAL(int,int)),winTVScreen,SLOT(changeAudioVolumeSLOT(int,int)));

  // connect brightness, contrast, colour, hue
  connect(options,SIGNAL(newS1ValueSIGNAL(int)),winTVScreen,SLOT(changeBrightnessSLOT(int)));
  connect(options,SIGNAL(newS2ValueSIGNAL(int)),winTVScreen,SLOT(changeContrastSLOT(int)));
  connect(options,SIGNAL(newS3ValueSIGNAL(int)),winTVScreen,SLOT(changeColorSLOT(int)));
  connect(options,SIGNAL(newS4ValueSIGNAL(int)),winTVScreen,SLOT(changeHueSLOT(int)));

  // connect tune, input, channel 
  connect(selector,SIGNAL(frequencyChangedSIGNAL(int)),winTVScreen,SLOT(changeFrequencySLOT(int)));
  connect(selector,SIGNAL(inputChangedSIGNAL(int)),winTVScreen,SLOT(changeSourceSLOT(int)));
  connect(selector,SIGNAL(normChangedSIGNAL(int)),winTVScreen,SLOT(changeNormSLOT(int)));

  // connect x11Events
  connect(this,SIGNAL(x11EventSIGNAL(XEvent *)),winTVScreen,SLOT(Shellx11EventSLOT(XEvent *)));

  // connect ctab dialog apply, cancel, default
  connect(tab,SIGNAL(applyButtonPressed()),this,SLOT(applyTabSLOT())); 
  connect(tab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelTabSLOT())); 
  connect(tab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultTabSLOT()));

  // connect cmtab dialog apply, cancel, default
  connect(mtab,SIGNAL(applyButtonPressed()),this,SLOT(applyMtabSLOT())); 
  connect(mtab,SIGNAL(cancelButtonPressed()),this,SLOT(cancelMtabSLOT())); 
  connect(mtab,SIGNAL(defaultButtonPressed()),this,SLOT(defaultMtabSLOT()));

  // use volume given by mixer or volume given by rc-file ??
  if (rc->use_master_volume) {
    rc->volume_left=mix->getVolume(VOL_LEFT);
    rc->volume_right=mix->getVolume(VOL_RIGHT);
  } 

  // mute/unmute sound
  if (!rc->sound_on) {
    // mute
    mix->setVolume(0,VOL_BOTH);
    emit audioMuteToggledSIGNAL();
  } else {
    mix->setVolume(rc->volume_left,VOL_LEFT);
    mix->setVolume(rc->volume_right,VOL_RIGHT);
  }

  QString number;
  number.setNum(rc->volume_left);number.append("%");
  statusbar->changeItem((const char*)number,SB_VOL);

  // some default settings
  isfullscreen=false;
  isfullsize=false;
  capture=rc->capture;
  fixed=false;
  keyup=false;
  lastCutCopy=0;
  copyChannel=false;

#ifdef mainDEBUG
  debug("main: end of init");
#endif
}

// =================================================================================================

winTVMainWidget::~winTVMainWidget()
{
#ifdef mainDEBUG
    debug("main: destroying winTVMainWidget");
#endif

  if ( rc->ol_channel != -1 ) rc->ol_channel= channel_list.at();
  if (rc->autosave) saveChannels((const char *)(rc->dataDir+QString("/")+rc->channelListName));

#if 0
  QRect rect= frameGeometry();
  rc->ol_geometry.setX(rect.x());
  rc->ol_geometry.setY(rect.y());
  rc->ol_geometry.setWidth(width());
  rc->ol_geometry.setHeight(height());
#else
  rc->ol_geometry= frameGeometry();
#endif
  rc->ol_toolbar= toolbar->isVisible();
  rc->ol_menubar= menubar->isVisible();
  rc->ol_statusbar= statusbar->isVisible();

  /*
  int i;
  for (i = 0; i < vm_count; i++) {
    free(vm_modelines[i]);
  }
  //  free(vm_modelines);
  */

#ifdef mainDEBUG
  debug("main: ~winTVMainWidget: deleting classes");
#endif

  delete mix;

  delete selector;
  delete options;
  //delete freq;
  delete tab;
  
  delete mixdev;
  delete aspectratio;
  delete snapshot;
  delete general;
  delete mtab;

  delete tvmenu;
  delete smenu;
  delete chmenu;
  delete optmenu;

  delete toolbar;
  delete menubar;

  delete ircCodes;

#ifdef mainDEBUG
    debug("main: ~winTVMainWidget: deleting winTVScreenWidget");
#endif

  delete winTVScreen;

#ifdef mainDEBUG
    debug("main: winTVMainWidget destroyed");
#endif
}

// =================================================================================================

void winTVMainWidget::scanChannelsSLOT()
{
#ifdef mainDEBUG
  debug("main: winTVMainWidget::scanChannelsSLOT");
#endif
    
  csan = new winTVcsan(this,"scanner",rc,
		       winTVScreen->getCapCap(),
		       winTVScreen->getCapChannel(),
		       winTVScreen->getCapTuner(),
		       winTVScreen->getCapAudio(),
		       winTVScreen);

  connect(csan,SIGNAL(readyScanChannelsSIGNAL(int)),this,SLOT(readyScanChannelsSLOT(int)));
  connect(csan,SIGNAL(frequencyChangedSIGNAL(int)),winTVScreen,SLOT(changeFrequencySLOT(int)));
  connect(csan,SIGNAL(inputChangedSIGNAL(int)),winTVScreen,SLOT(changeSourceSLOT(int)));
  connect(csan,SIGNAL(normChangedSIGNAL(int)),winTVScreen,SLOT(changeNormSLOT(int)));

  csan->setDataSLOT(&channel_list);
  csan->show();
  
}

// =================================================================================================

void winTVMainWidget::readyScanChannelsSLOT( int button )
{
#ifdef mainDEBUG
    debug("main: winTVMainWidget::readyScanChannelsSLOT");
#endif

  disconnect(csan);
  csan->hide();

  // get data
  if ( button ) {
    QList<channel> *clist = csan->getData();
    channel *ch = clist->first();
    while (ch) {
      channel_list.append(ch);
      ch= clist->next();
    }
    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
  }

  emit channelChangedSIGNAL();
  delete  csan;
}

// =================================================================================================

void winTVMainWidget::audioModeChangedSLOT(int mode)
{
  channel *selchnl= channel_list.current();
  if (mode & AUDIOMODE_AUTODETECT) {
    setCaption((const char*)(QString("kWinTV - ")+QString(selchnl->name)
			     +QString(" - (auto) ")
			     +QString(audiomode[mode ^ AUDIOMODE_AUTODETECT].str)));
  } else {
    setCaption((const char*)(QString("kWinTV - ")+QString(selchnl->name)
			     +QString(" - ")+QString(audiomode[mode].str)));
  }
}

// =================================================================================================

void winTVMainWidget::channelChangedSLOT()
{
  if ( channel_list.at() == -1 ) return;

  channel *ch= channel_list.current();
  setCaption((const char*)(QString("kWinTV - ")+QString(ch->name)
			   +QString(" - ")+QString(audiomode[winTVScreen->getAudioMode()].str)));
  lcd->display( channel_list.at() );
  
  winTVScreen->changeChannelValuesSLOT( ch, channel_list.at() );
}

// =================================================================================================

void winTVMainWidget::keyPressedSLOT(int key)
{
  warning(i18n("main: key pressed in tv screen, key: %d"), key);
}

// =================================================================================================
void winTVMainWidget::fullSizeSLOT()
{
#ifdef mainDEBUG
  debug(i18n("main: set size to %s"),isfullsize?"full":"normal");
#endif
  
  //winTVScreen->changeCaptureSLOT(0);
  if (isfullsize) {
    // old size with frame etc.
    resize(oldPos.width(),oldPos.height());
    tvmenu->setItemChecked(tvmid[2],false);
    tvmenu->setItemEnabled(tvmid[1],true);
  } else {
    // full size
    int Yoffset=0,Xoffset=0;
    if (toolbar->isVisible()) {          // FIX ME
      if ((toolbar->barPos() == KToolBar::Left) || (toolbar->barPos() == KToolBar::Right))
	Xoffset+=toolbar->width();
      if((toolbar->barPos() == KToolBar::Top) || (toolbar->barPos() == KToolBar::Bottom))
	Yoffset+=toolbar->height();
    }
    if (menubar->isVisible()) { 
      Yoffset+=menubar->height();
    }
    if (statusbar->isVisible()) { 
      Yoffset+=statusbar->height();
    }
    oldPos=geometry();
    resize(winTVScreen->win_max_width()+5+Xoffset,winTVScreen->win_max_height()+5+Yoffset);

    tvmenu->setItemChecked(tvmid[2],true);
    tvmenu->setItemEnabled(tvmid[1],false);
  }
  isfullsize=!isfullsize;
}
 
// =================================================================================================

void winTVMainWidget::fullScreenSLOT()
{
  isfullscreen=!isfullscreen;

#ifdef mainDEBUG
  debug(i18n("main: set screen size to %s"),isfullscreen?"full":"normal");
#endif
  
  //winTVScreen->changeCaptureSLOT(0);
  if (!isfullscreen) {
    // old size with frame, menus etc.

#ifdef HAVE_LIBXXF86VM
    if (have_vm) {
      if (rc->fs_switch_res && res_was_switched) {
#ifdef mainDEBUG
	debug("main: switching back to modeline %d, %d x %d",
	      0,vm_modelines[0]->hdisplay,vm_modelines[0]->vdisplay);
#endif
	XF86VidModeSwitchToMode(qt_xdisplay(),XDefaultScreen(qt_xdisplay()),vm_modelines[0]);
	res_was_switched= false;
      }
    }
#endif HAVE_LIBXXF86VM

    winTVScreen->showChannelInfoEnableSLOT( 0 );

    if ( !optmenu->isItemChecked(omid[ID_HIDEALL]) ) {
      menubar->show();
      if ( !optmenu->isItemChecked(omid[ID_HIDETOOL]) ) toolbar->show();
      if ( !optmenu->isItemChecked(omid[ID_HIDESTATUS]) ) statusbar->show();
    }
    //updateRects();

    setGeometry(oldPos);
    //resize(oldPos.width(),oldPos.height());
    //move(oldPos.x(),oldPos.y());

    updateRects();

    if ( optmenu->isItemChecked(omid[ID_HIDEMOUSE]) ) winTVScreen->toggleMouseCursorSLOT();

    tvmenu->setItemChecked(tvmid[1],false);
    tvmenu->setItemEnabled (tvmid[2],true);
  } else {
    // switch resolution to get full screen size
    oldPos=geometry();

    toolbar->hide();
    menubar->hide();
    statusbar->hide();
    //updateRects();

    winTVScreen->showChannelInfoEnableSLOT( rc->fs_infooverlay );

#ifdef HAVE_LIBXXF86VM
    int i;
    if (have_vm) {
      /*
      for (i = 0; i < vm_count; i++) {
	free(vm_modelines[i]);
      }
      //free(vm_modelines);
      */
      XF86VidModeGetAllModeLines(dpy,XDefaultScreen(dpy),&vm_count,&vm_modelines);

      // get the number of the actual modeline
      for (i = 0; i < vm_count; i++) {
	if (rc->fs_x == vm_modelines[i]->hdisplay && rc->fs_y == vm_modelines[i]->vdisplay)
	  break;
      }

      res_was_switched=false;

      if (rc->fs_switch_res && i != 0 && i != vm_count) {
#ifdef mainDEBUG
	debug("main: switching down to modeline %d, %d x %d",
	      i,vm_modelines[i]->hdisplay,vm_modelines[i]->vdisplay);
#endif
	
	XF86VidModeSwitchToMode(qt_xdisplay(),XDefaultScreen(qt_xdisplay()),vm_modelines[i]);

	res_was_switched= true;
      } else {
	res_was_switched=false;
#ifdef mainDEBUG
	debug("main: no modeline switching!");
#endif
      }
#endif HAVE_LIBXXF86VM

      QWidget *d = kapp->desktop();
      if (rc->fs_upper_left) {
	setGeometry(0,0,winTVScreen->win_max_width(),winTVScreen->win_max_height());
      } else {
	QRect geom= geometry();
	QRect fgeom= frameGeometry();
	setGeometry(fgeom.x()-geom.x(),fgeom.y()-geom.y(),d->width(),d->height());
      }

#ifdef HAVE_LIBXXF86VM
      if (have_vm && res_was_switched) {
	XF86VidModeSetViewPort(qt_xdisplay(),XDefaultScreen(qt_xdisplay()),
			       (kapp->desktop()->width()-winTVScreen->win_max_width())/2,
			       (kapp->desktop()->height()-winTVScreen->win_max_height())/2);
      }
#endif HAVE_LIBXXF86VM

      QCursor::setPos(kapp->desktop()->width()/2,kapp->desktop()->height()/2);
      if ( !optmenu->isItemChecked(omid[ID_HIDEMOUSE]) ) winTVScreen->toggleMouseCursorSLOT();

debug("main: c x: %d, c y: %d",kapp->desktop()->width()/2,kapp->desktop()->height()/2);
    } else {
      // we dont use the video mode extension at all
      QWidget *d = kapp->desktop();
      if (rc->fs_upper_left) {
	setGeometry(0,0,winTVScreen->win_max_width(),winTVScreen->win_max_height());
      } else { 
	QRect geom= geometry();
	QRect fgeom= frameGeometry();
	setGeometry(fgeom.x()-geom.x(),fgeom.y()-geom.y(),d->width(),d->height());
      }
      if ( !optmenu->isItemChecked(omid[ID_HIDEMOUSE]) ) winTVScreen->toggleMouseCursorSLOT();
    } // have_vm

    tvmenu->setItemChecked(tvmid[1],true);
    tvmenu->setItemEnabled(tvmid[2],false);
  } // isfullscreen
}
 
// =================================================================================================

void winTVMainWidget::adjustAspectRatioSLOT()
{
  int ar=rc->aspectRatio;

  if ((ar == ASPECT_NONE) || !isVisible()) return;

  fixed=true;

  int xa,ya;
  int Xoffset=0,Yoffset=0;

  switch (ar)
    {
    case ASPECT_4_3:
      xa=4; ya=3;
      break;
    case ASPECT_16_9: 
      xa=16; ya=9;
      break;
    default:
      warning(i18n("main: incorrect value for aspect ratio: %d"),ar);
      return;
    }

  if ((menubar->menuBarPos() != KMenuBar::Floating) && (menubar->isVisible())) {
    Yoffset+=menubar->height();
  }

  if (statusbar->isVisible()) {
    Yoffset+=statusbar->height();
  }

  if ((toolbar->barPos() != KToolBar::Floating) && (toolbar->isVisible())) {
    if ((toolbar->barPos() == KToolBar::Left) || (toolbar->barPos() == KToolBar::Right))
      Xoffset+=toolbar->width();
    if((toolbar->barPos() == KToolBar::Top) || (toolbar->barPos() == KToolBar::Bottom))
      Yoffset+=toolbar->height();
  }
  
  // aktuelle Breite u Hoehe der tv screen
  QRect size=winTVScreen->geometry();

  int h=(ya*(size.width()))/xa;
  int w=(xa*(size.height()))/ya;
  int sh=size.height();
  int sw=size.width(); 

  if (sh < h) {
    size.setHeight(sh + ((h - sh) / 2) );
    size.setWidth(xa*(size.height())/ya);
    resize(size.width()+Xoffset,size.height()+Yoffset);
  } else 
    if (sw < w) {
      size.setWidth(sw + ((w - sw) / 2) );
      size.setHeight((ya*(size.width()))/xa);
      resize(size.width()+Xoffset,size.height()+Yoffset);
    } 

#ifdef mainDEBUG
  debug("main: adjusted aspect ratio");
#endif
}

// =================================================================================================

void winTVMainWidget::buttonsSLOT(int b)
{
#ifdef mainDEBUG
  //debug(i18n("main: button %d klicked"),b);
#endif

  if ( !winTVScreen ) return;

  switch (b) {
  case BID_DOWN: // switch channel down
    channelDownSLOT();
    break;
  case BID_UP:   // switch channel up
    channelUpSLOT();
    break;
  case BID_CAP:  // toggle capture 
    captureToggleSLOT();
    break;
  case BID_SS:   // make a snap shot
    screenShotSLOT();
   
    break;
  case BID_SC:   // show window with channel list and volume control ???
    toggleChannelBoxSLOT();
    break;
  case BID_EXIT: // exit
    kapp->quit();
    break;
  case BID_FAR:  // adjust aspect ratio
    adjustAspectRatioSLOT();
    break;
  case BID_VUP:  // volume up
    volumeUpSLOT();
    break;
  case BID_MUTE: // toggle mute/unmute
    soundSLOT();
    break;
  case BID_VDOWN:// voluem down
    volumeDownSLOT();
    break;
  case BID_FSS:  // toggle full screen
    fullScreenSLOT();
    break;
  default:       // this should never happen...
    fatal("main: hardcore error 00004");
  }
}


// =============================================================================== 

void winTVMainWidget::toggleChannelBoxSLOT()
{
  if ( !rc->autoplaceChannelBox ) emit toggleChannelBoxSIGNAL( NULL );

  QPoint to;
  QRect rect= frameGeometry();

#ifdef mainDEBUG
  debug(i18n("main: autoplace to %d"),rc->posOfChannelBox);
#endif
  
  switch ( rc->posOfChannelBox ) {
  case UPPER_LEFT:
    emit toggleChannelBoxSIGNAL( NULL );
    break;
  case LOWER_LEFT:
    emit toggleChannelBoxSIGNAL( NULL );
    break;
  case UPPER_RIGHT:
    to.setX(rect.width()+rect.x()+5);
    to.setY(rect.y());
#ifdef mainDEBUG
  debug(i18n("main: autoplace to %d,%d"),to.x(),to.y());
#endif
    emit toggleChannelBoxSIGNAL( &to );
    break;
  case LOWER_RIGHT:
    emit toggleChannelBoxSIGNAL( NULL );
    break;
  default:
    emit toggleChannelBoxSIGNAL( NULL );
  }
}

// =============================================================================== 

void winTVMainWidget::mousecursorToggleSLOT()
{
  winTVScreen->toggleMouseCursorSLOT();
  optmenu->setItemChecked(omid[ID_HIDEMOUSE],!winTVScreen->getCursorState());
}

// =============================================================================== 

void winTVMainWidget::captureToggleSLOT()
{
  capture=!capture;
  tvmenu->setItemChecked(tvmid[0],capture);
  emit captureToggleSIGNAL(capture);
}

// =============================================================================== 

void winTVMainWidget::screenShotSLOT()
{
  if (!capture) 
    QMessageBox::information( this, 
			      (const char*)(QString("kWinTV ")+QString(VERSION)),
			      i18n("Capture has to be set on to make snapshots!"));

  winTVScreen->snapShotSLOT((const char *)rc->ss_prefix,
			    rc->ss_fixed,
			    rc->ss_width,rc->ss_height,
			    rc->ss_format);
  QString number;
  number.setNum(winTVScreen->getShotNumber()+1);
  statusbar->changeItem((const char*)number,SB_SNAP);
}

// =============================================================================== 

void winTVMainWidget::videoClipSLOT()
{
  QString propstr(i18n("\n PLEASE READ THIS INFORMATION! \n\n"
		   "There is a bug in kwintv which causes the program\n"
		   "to crash if you move the mouse cursor out of the \n"
		   "window titled \"kWinTV - Video Clip\"!\n"
		   "Dont wonder if there is no picture while grabbing\n"
		   "a video clip... You can only grab to screen or to\n"
		   "memory...\n"));

  QMessageBox::information( this, 
			    (const char*)(QString(i18n("kWinTV "))+QString(VERSION)),
                            (const char*)propstr);

  emit toggleVClipSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::startVClipSLOT(struct MOVIE_PARAMS *par)
{
  if (!capture) 
    QMessageBox::information( this, 
			      (const char*)(QString("kWinTV ")+QString(VERSION)),
			      i18n("Capture has to be set on to grab a video clip!"));
  
#ifdef withTVscreen
  if (capture) winTVScreen->startVideoClipSLOT(par);
#endif
}

// =============================================================================== 

void winTVMainWidget::endVClipSLOT()
{
#ifdef withTVscreen
  if (capture) winTVScreen->endVideoClipSLOT();
#endif
}

// =============================================================================== 

void winTVMainWidget::frameVClipSLOT(int c)
{
  emit frameVClipSIGNAL(c);
}

// =============================================================================== 

void winTVMainWidget::statVideoClipFrameSLOT(double d)
{
  emit statVideoClipFrameSIGNAL(d);
}

// =============================================================================== 

void winTVMainWidget::framelostVClipSLOT(int c)
{
  emit framelostVClipSIGNAL(c);
}

// =============================================================================== 

void winTVMainWidget::setupSLOT()
{
  //if (!rc->sound_on) soundSLOT();
  mixdev->setData(rc);
  aspectratio->setData(rc);
  snapshot->setData(rc);
  general->setData(rc);
  //#ifdef HAVE_LIBXXF86VM
  fullscreen->setData(rc);
  //#endif
  mtab->show();
}

// =============================================================================== 

void winTVMainWidget::userKeysConfigSLOT()
{
  userKeys->configureKeys(this);
}

// =============================================================================== 

void winTVMainWidget::ircCodesConfigSLOT()
{
  ircCodes->configureCodes(this);
}

// =============================================================================== 

void winTVMainWidget::audioAutoSLOT()
{
  emit audioModeChangedSIGNAL(0);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i], (winTVScreen->getAudioMode()+1) == (i+1));
}

// =============================================================================== 

void winTVMainWidget::audioMonoSLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_MONO);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::audioStereoSLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_STEREO);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::audioL1SLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_LANG1);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::audioL2SLOT()
{
  emit audioModeChangedSIGNAL(VIDEO_SOUND_LANG2);
  for (int i=0; i < 5; i++ )
    smenu->setItemChecked(smid[i],winTVScreen->getAudioMode() == i);
}

// =============================================================================== 

void winTVMainWidget::addChannelSLOT()
{
  //debug("main: att channel, channel count %d",selchnl->count());
  if ( !channel_list.isEmpty() ) {
    options->setDataSLOT(channel_list.current());
    selector->setDataSLOT(channel_list.current());
  } else {
    //options->setDataSLOT(selchnl);
    fatal("main: hardcore error 00003");
  }
  //if (!winTVScreen->getCaptureState()) captureToggleSLOT();

  wasEdit=false;
  tab->setCaption("kWinTV - Add a new program");
  tab->show();
}

// =============================================================================== 

void winTVMainWidget::editChannelSLOT()
{
  //if (!winTVScreen->getCaptureState()) captureToggleSLOT();

  options->setDataSLOT( channel_list.current() );
  selector->setDataSLOT( channel_list.current() );

  wasEdit=true;
  tab->setCaption((const char *)(QString("kWinTV - Edit program <")
				 +QString(channel_list.current()->name)+QString(">")));
  tab->show();
}

// =============================================================================== 

void winTVMainWidget::deleteChannelSLOT()
{
  channel *newchannel;

  if ( channel_list.isEmpty() ) return;

#ifdef mainDEBUG
  debug("main: deleting channel %d, %s",channel_list.at(),channel_list.current()->name);
#endif
  if ( lastCutCopy != NULL ) delete lastCutCopy;
  lastCutCopy= channel_list.current();
  channel_list.remove();
  copyChannel=false;

  if ( channel_list.isEmpty() ) {
    newchannel= new channel(rc->freq,"<no channel>",rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in);
    channel_list.append( newchannel );
#ifdef mainDEBUG
    debug("main: new nonexistent channel %d, %s",channel_list.at(),channel_list.current()->name);
#endif
  }

#ifdef mainDEBUG
  debug("main: new aktual channel %d, %s",channel_list.at(),channel_list.current()->name);
#endif
  emit newChannelListSIGNAL( &channel_list );
  emit channelChangedSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::copyChannelSLOT()
{
#ifdef mainDEBUG
  debug("main: copy channel %d, %s",channel_list.at(),channel_list.current()->name);
#endif
  if ( lastCutCopy != NULL ) delete lastCutCopy;
  lastCutCopy= channel_list.current();
  copyChannel=true;
}

// =============================================================================== 

void winTVMainWidget::pasteChannelSLOT()
{
  //channel_list.setAutoDelete ( !channel_list.autoDelete ()  );

  channel *cfrom= lastCutCopy;
  channel *cto= channel_list.current();

  if (!copyChannel) channel_list.remove( channel_list.find(cfrom) );
  channel_list.insert( channel_list.find(cto), cfrom );

  //channel_list.setAutoDelete ( !channel_list.autoDelete ()  );

  emit newChannelListSIGNAL( &channel_list );
  emit channelChangedSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::cancelTabSLOT()
{
#ifdef withTVscreen
  emit channelChangedSIGNAL();
#endif
}

// =============================================================================== 

void winTVMainWidget::defaultTabSLOT()
{
  if ( !channel_list.isEmpty() ) {
    options->setDataSLOT( channel_list.current());
    selector->setDataSLOT(channel_list.current());
  } else {
    //options->setDataSLOT(channel_list.current());
  }
  
  // call winTVscreen-Widget with original settings
#ifdef withTVscreen
  emit channelChangedSIGNAL();
#endif
}

// =============================================================================== 

void winTVMainWidget::applyTabSLOT()
{
  if (wasEdit) {
    channel *selchnl= channel_list.current();
    selchnl->startwiththis=selector->getData()->startwiththis;
    int act=  channel_list.at();
    if (selchnl->startwiththis) {
      channel *n= channel_list.first();
      while ( n ) {
	if ( n != selchnl ) {
	  n->startwiththis=0;
	}
	n= channel_list.next();
      }
    }
    channel_list.at(act);
    // copy new values into channel
    selchnl->frequency=selector->getData()->frequency;
    selchnl->input=selector->getData()->input;
    selchnl->norm=selector->getData()->norm;
    selchnl->brightness=options->getData()->brightness;
    selchnl->contrast=options->getData()->contrast;
    selchnl->color=options->getData()->color;
    selchnl->hue=options->getData()->hue;
    selchnl->channelId=selector->getData()->channelId;

    // change name if changed
    if (strcmp(selchnl->name,selector->getData()->name)) {
      if (selchnl->name) delete selchnl->name;
      selchnl->name= new char[strlen(selector->getData()->name)+1];
      strcpy(selchnl->name,selector->getData()->name);
      emit newChannelListSIGNAL( &channel_list );
    }

    emit channelChangedSIGNAL();
  } else {
    // add new channel

    //debug("new number %d, %d",selchnl->number,selchnl->count());
    channel *selchnl= new channel(selector->getData()->frequency,
				  selector->getData()->name,
				  options->getData()->color,
				  options->getData()->brightness,
				  options->getData()->hue,
				  options->getData()->contrast,
				  selector->getData()->norm,
				  selector->getData()->input);
    selchnl->channelId=selector->getData()->channelId;
    int act=  channel_list.at();
    //debug("new number %d",n->number);
    selchnl->startwiththis=selector->getData()->startwiththis;
    if (selchnl->startwiththis) {
      channel *n= channel_list.first();
      while ( n ) {
	if ( n != selchnl ) {
	  n->startwiththis=0;
	}
	n= channel_list.next();
      }
    }
    channel_list.at(act);
    if ( !channel_list.insert( channel_list.at(), selchnl ))
	 warning("main: channel_list.insert == false");

    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
  }
}

// =============================================================================== 

void winTVMainWidget::applyMtabSLOT()
{
#ifdef mainDEBUG
  debug("main: applyMtabSLOT omixdev");
#endif
  // omixdev
  rc->volume_right=mixdev->getData()->volume_right;
  rc->use_tv_volume=mixdev->getData()->use_tv_volume;
  rc->use_master_volume=mixdev->getData()->use_master_volume;
  rc->sliders_att_startup=mixdev->getData()->sliders_att_startup;
  rc->mixerDevice=mixdev->getData()->mixerDevice;
#ifdef mainDEBUG
  debug("main: applyMtabSLOT oscreenshot");
#endif
  // oscreenshot
  rc->ss_fixed=snapshot->getData()->ss_fixed;
  rc->ss_width=snapshot->getData()->ss_width;
  rc->ss_height=snapshot->getData()->ss_height;
  rc->ss_format=snapshot->getData()->ss_format;
  rc->ss_prefix=QString(snapshot->getPrefix());
#ifdef mainDEBUG
  debug("main: applyMtabSLOT oaspectratio");
#endif
  // oaspectratio
  rc->autoar=aspectratio->getData()->autoar;
  rc->aspectRatio=aspectratio->getData()->aspectRatio;
#ifdef mainDEBUG
  debug("main: applyMtabSLOT ogeneral; rc->ol_channel = %d",general->getData()->ol_channel);
#endif
  // ogeneral
  rc->capture=general->getData()->capture;
  rc->autosave=general->getData()->autosave;
  rc->screensaver=general->getData()->screensaver;
  rc->ol_restore=general->getData()->ol_restore;
  rc->posOfChannelBox=general->getData()->posOfChannelBox;
  rc->autoplaceChannelBox=general->getData()->autoplaceChannelBox;
  rc->ol_channel=general->getData()->ol_channel;
#ifdef mainDEBUG
  debug("main: applyMtabSLOT fullscreen");
#endif
  // fullscreen
  rc->fs_res=fullscreen->getData()->fs_res;
  rc->fs_infooverlay=fullscreen->getData()->fs_infooverlay;
  rc->fs_switch_res=fullscreen->getData()->fs_switch_res;
#ifdef HAVE_LIBXXF86VM
  rc->fs_x=vm_modelines[rc->fs_res]->hdisplay;
  rc->fs_y=vm_modelines[rc->fs_res]->vdisplay;
#endif

#ifdef mainDEBUG
  debug("main: applyMtabSLOT resetting sound");
#endif
  if (rc->sound_on) {
    mix->setVolume(rc->volume_left,VOL_LEFT);
    mix->setVolume(rc->volume_right,VOL_RIGHT);
  } else {
    mix->setVolume(0,VOL_BOTH);
  }
  if (rc->autoar) adjustAspectRatioSLOT();
#ifdef mainDEBUG
  debug("main: applyMtabSLOT end");
#endif
}

// =============================================================================== 

void winTVMainWidget::defaultMtabSLOT()
{
  mixdev->setData(rc);
  snapshot->setData(rc);
  aspectratio->setData(rc);
  general->setData(rc);
  fullscreen->setData(rc);
}

// =============================================================================== 

void winTVMainWidget::cancelMtabSLOT()
{
  if (rc->sound_on) {
    mix->setVolume(rc->volume_left,VOL_LEFT);
    mix->setVolume(rc->volume_right,VOL_RIGHT);
  } else {
    mix->setVolume(0,VOL_BOTH);
  }
}

// =============================================================================== 

void winTVMainWidget::saveChannelsSLOT()
{
  saveChannels((const char*)(rc->dataDir+QString("/")+rc->channelListName));
#ifdef COMMIT
  QMessageBox::information( this, 
			    (const char*)(QString("kWinTV ")+QString(VERSION)),
			    i18n("Programs saved!"));
#endif
}

// =============================================================================== 

void winTVMainWidget::saveChannelsAsSLOT()
{
  QString fileName = 
    QFileDialog::getSaveFileName(rc->dataDir,"*.ch",this,filename);
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    saveChannels(filename); 
    rc->channelListName=fileName.mid(fileName.findRev('/')+1,fileName.length());
    statusbar->changeItem((const char*)rc->channelListName,SB_PROG);
#ifdef COMMIT
    QMessageBox::information( this, 
			      (const char*)(QString("kWinTV ")+QString(VERSION)),
			      i18n("Programs saved!"));
#endif
  }
}

// =============================================================================== 

void winTVMainWidget::loadChannelsSLOT()
{
  QString fileName = 
    QFileDialog::getOpenFileName(rc->dataDir,"*.ch",this);
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    loadChannels(filename); 
    rc->channelListName=fileName.mid(fileName.findRev('/')+1,fileName.length());
    statusbar->changeItem((const char*)rc->channelListName,SB_PROG);
  }
}

// =============================================================================== 

void winTVMainWidget::importXAWTVChannelsSLOT()
{
  char tag[32], val[100];
  char line[255];
  country cntry;

  QString fileName = QFileDialog::getOpenFileName(QDir::homeDirPath (),"*xawtv",this);

  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);

    QFile fin(filename);

    if (!fin.open(IO_ReadOnly)) {
      warning(i18n("main: Can't load channel-filename %s!"),filename);
      if ( channel_list.isEmpty() ) {
	channel *s= new channel(rc->freq,"<no channel>",rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in);
	channel_list.append( s );
      }
      //emit newChannelListSIGNAL(&channel_head);
      //if (selchnl != NULL) emit channelChangedSIGNAL(selchnl);
      return;
    }

    QTextStream t( &fin ); 

    channel *n;
    // remove existing channels 
    channel_list.clear();

    int thenorm=DNORM;
    int theinput=DIN;

    strcpy(line,(const char *)t.readLine());
    while(line[0] != '[' ) {
debug("xawtv: line: <%s>",line);
      if ((strlen(line)==0) || line[0] == '#' || line[0] == '%') {
	strcpy(line,(const char *)t.readLine());
	continue;
      }
      if (2 != sscanf(line," %31[^= ] = %99[^\n]",tag,val)) {
	warning("main: xawtv: parse error %s:%s in %s\n",tag,val,filename);
      } 

debug("xawtv: scan: %s:%s",tag,val);

      if ( 0 == strcmp(tag,"freqtab")) {
        cntry.change(val);
debug("xawtv: country %s %s",tag,val);
      } else if ( 0 == strcmp(tag,"source") ) {
	theinput= atoi(val);
      } else if (0 == strcmp(tag,"norm")) {
	thenorm= atoi(val);
      } else if (0 == strcmp(tag,"fullscreen")) {
	warning("main: scan xawtv: tag <%s> ignored!",tag);
      } else if (0 == strcmp(tag,"mixer")) {
	warning("main: scan xawtv: tag <%s> ignored!",tag);
      }
      strcpy(line,(const char *)t.readLine());
    }

debug("xawtv: channel scan");
    while(!t.eof()) {
      n= new channel(0,"<no name>",DCOL,DBRI,DHUE,DCONTRAST,thenorm,theinput);
      
      if (n->name) delete(n->name);
      n->name=new char[strlen(line)+1];
      strncpy(n->name,&line[1],strlen(line)-1);
      n->name[strlen(line)-2]=0;
debug("xawtv: channel name: %s, %s",n->name,line);

      strcpy(line,(const char *)t.readLine());
      while (!t.eof() && (line[0] != '[')) {
	if ((strlen(line)==0) || line[0] == '#' || line[0] == '%') {
	  strcpy(line,(const char *)t.readLine());
	  continue;
	}
	if (2 != sscanf(line," %31[^= ] = %99[^\n]",tag,val)) {
	  warning("main: xawtv: parse error <%s>:<%s> in %s\n",tag,val,filename);
	}
debug("xawtv: scan: %s:%s",tag,val);
	if (0 == strcmp(tag,"key")) { 
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	} else if (0 == strcmp(tag,"capture")) {
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	} else if (0 == strcmp(tag,"source")) {
	  n->input = 0;
	} else if (0 == strcmp(tag,"norm")) {
	  n->norm = 0;
	} else if (0 == strcmp(tag,"channel")) {
	  n->frequency = cntry.cftofreq(val,0);
debug("xawtv: frequency <%s>, %d",val,n->frequency);
	} else if (0 == strcmp(tag,"fine")) {
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	} else if (0 == strcmp(tag,"color"))
	  n->color = atoi(val);
	else if (0 == strcmp(tag,"bright"))
	  n->brightness = atoi(val);
	else if (0 == strcmp(tag,"hue"))
	  n->hue = atoi(val);
	else if (0 == strcmp(tag,"contrast"))
	  n->contrast = atoi(val);
	else if (0 == strcmp(tag,"freqtab"))
	  warning("main: scan xawtv: tag <%s> ignored!",tag);
	strcpy(line,(const char *)t.readLine());
      }

#ifdef mainDEBUG
      debug("main: channel: %s, %d, %d",n->name, n->frequency, n->frequency*100/16);
#endif
      channel_list.append( n );
    }

    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();

  }
}

// =============================================================================== 

void winTVMainWidget::importXTVSCREENChannelsSLOT()
{
  QString fileName = QFileDialog::getOpenFileName(QDir::homeDirPath (),"*.*",this,".xtvrc");
  if ( !fileName.isNull() ) {                 // got a file name
    strcpy(filename,fileName);
    loadChannels(filename,true); 
  }
}

// =============================================================================== 

void winTVMainWidget::vtxSLOT()
{
  QMessageBox::information( this, 
			    (const char*)(QString(i18n("VTX for kWinTV "))+QString(VERSION)),
                            i18n("Not aviable!"));
}

// =============================================================================== 

void winTVMainWidget::naSLOT()
{
  QMessageBox::information( this, 
			    (const char*)(QString("kWinTV ")+QString(VERSION)),
                            i18n("This function is not available in this version!"));

}

// =============================================================================== 

void winTVMainWidget::saveRCSLOT()
{
  if ( rc->ol_channel != -1 ) rc->ol_channel= channel_list.at();
#if 0
  QRect rect= frameGeometry();
  rc->ol_geometry.setX(rect.x());
  rc->ol_geometry.setY(rect.y());
  rc->ol_geometry.setWidth(width());
  rc->ol_geometry.setHeight(height());
#else
  rc->ol_geometry= frameGeometry();
#endif
  rc->ol_toolbar= toolbar->isVisible();
  rc->ol_menubar= menubar->isVisible();
  rc->ol_statusbar= statusbar->isVisible();

  if (rc->save()) 
    QMessageBox::warning( this, 
			  (const char*)(QString("kWinTV ")+QString(VERSION)),
			  i18n("kWinTV was not able to save rc data!"));
#ifdef COMMIT
  else 
    QMessageBox::information( this, 
			      (const char*)(QString("kWinTV ")+QString(VERSION)),
			      i18n("General options saved!"));
#endif
}

// =============================================================================== 

void winTVMainWidget::toggleMenuBarSLOT()
{ 
#ifdef mainDEBUG  
  debug("toggleMenuBarSLOT");
#endif
  int offset;
  QRect size=geometry();
  if (menubar->isVisible()) {
    offset=-menubar->height();
    menubar->hide();
  } else {
    menubar->show();
    offset=-menubar->height();
  }
  resize(size.width(),size.height()+offset);
}

// =============================================================================== 

void winTVMainWidget::toggleStatusBarSLOT()
{ 
#ifdef mainDEBUG  
  debug("toggleStatusBarSLOT");
#endif
  int offset;
  QRect size=geometry();
  if (statusbar->isVisible()) {
    offset=-statusbar->height();
    statusbar->hide();
  } else {
    statusbar->show();
    offset=statusbar->height();
  }
  resize(size.width(),size.height()+offset);
  optmenu->setItemChecked(omid[ID_HIDESTATUS],!statusbar->isVisible());
}

// =============================================================================== 

void winTVMainWidget::toggleToolBarSLOT()
{
#ifdef mainDEBUG  
  debug("toggleToolBarSLOT");
#endif
  if ( (toolbar->barPos() == KToolBar::Floating) ) return;
  int xoffset=0,yoffset=0;
  QRect size=geometry();
  if (toolbar->isVisible()) {
    if ((toolbar->barPos() == KToolBar::Left) || (toolbar->barPos() == KToolBar::Right))
      xoffset-=toolbar->width();
    if((toolbar->barPos() == KToolBar::Top) || (toolbar->barPos() == KToolBar::Bottom))
      yoffset-=toolbar->height();
    toolbar->hide();
  } else {
    toolbar->show();
    if ((toolbar->barPos() == KToolBar::Left) || (toolbar->barPos() == KToolBar::Right))
      xoffset+=toolbar->width();
    if((toolbar->barPos() == KToolBar::Top) || (toolbar->barPos() == KToolBar::Bottom))
      yoffset+=toolbar->height();
  }
  resize(size.width()+xoffset,size.height()+yoffset);
    
  optmenu->setItemChecked(omid[ID_HIDETOOL],!toolbar->isVisible());
}

// =============================================================================== 

void winTVMainWidget::propertiesSLOT()
{
  int i;

  struct video_capability* vcap=winTVScreen->getCapCap();
  struct video_channel* vchan=winTVScreen->getCapChannel();
  struct video_audio* vaudio=winTVScreen->getCapAudio(); 
  struct video_tuner* vtuner=winTVScreen->getCapTuner();

  QString propstr(2000);
  QString dummy;

#ifdef mainDEBUG  
  debug("%s",vcap->name);
#endif
  propstr+=dummy.sprintf("Informations about your frame grabber card:\n\n");
  propstr+=dummy.sprintf("Name: %s\n\nCapabilities:\n\t",vcap->name);
  for (i = 0; device_cap[i] != NULL; i++)
    if (vcap->type & (1 << i))
      propstr+=dummy.sprintf("%s  ",device_cap[i]);
  
  propstr+=dummy.sprintf("\n\tmin. and max. size for TV screen: %dx%d => %dx%d",
			 vcap->minwidth,vcap->minheight,
			 vcap->maxwidth,vcap->maxheight);
  // channels 
  propstr+=dummy.sprintf("\n\nNumber of input channels: %d\n",vcap->channels);
  for (i = 0; i < vcap->channels; i++) {
    propstr+=dummy.sprintf("\t %s: %d %s %s%s%s\n",
			   vchan[i].name,
			   vchan[i].tuners,
			   (vchan[i].flags & VIDEO_VC_TUNER)   ? "tuner "  : "tuner ",
			   (vchan[i].flags & VIDEO_VC_AUDIO)   ? "audio "  : "",
			   (vchan[i].type & VIDEO_TYPE_TV)     ? "tv "     : "",
			   (vchan[i].type & VIDEO_TYPE_CAMERA) ? "camera " : "");
  }

  // tuner 
  propstr+=dummy.sprintf("\nType of tuner: %s \n\tRange: %lu - %lu MHz\n\tSupported norms: ",
			 vtuner->name,vtuner->rangelow,vtuner->rangehigh);
  for (i = 0; norms[i].str != NULL; i++) {
    if (vtuner->flags & (1<<i)) {
      propstr+=dummy.sprintf("%s ",norms[i].str);
    }
  }

  // audios
  propstr+=dummy.sprintf("\n\nNumber of audios: %d",vcap->audios);
  for (i = 0; i < vcap->audios; i++) {
    propstr+=dummy.sprintf("\n\t %d (%s): \n",i,vaudio[i].name);
    if (vaudio[i].flags & VIDEO_AUDIO_MUTABLE)
      propstr+=dummy.sprintf("\t\t mutable [%s]   ",
			     (vaudio[i].flags&VIDEO_AUDIO_MUTE) ? "muted":"unmuted");
    if (vaudio[i].flags & VIDEO_AUDIO_VOLUME)
      propstr+=dummy.sprintf("volume: %d ",vaudio[i].volume);
    else 
      propstr+=dummy.sprintf("noVol ");
    if (vaudio[i].flags & VIDEO_AUDIO_BASS)
      propstr+=dummy.sprintf("bass %d ",vaudio[i].bass);
    if (vaudio[i].flags & VIDEO_AUDIO_TREBLE)
      propstr+=dummy.sprintf("treble %d ",vaudio[i].treble);
    propstr+=dummy.sprintf("[%s]",audiomode[vaudio[i].mode].str);
  }

  QMessageBox::information( this, 
			    (const char*)(QString(i18n("Properties --- kWinTV "))+QString(VERSION)),
                            (const char*)propstr);

}

// =============================================================================== 

void winTVMainWidget::splitSlidersSLOT()
{
#ifdef mainDEBUG
  debug("main: splitSlidersSLOT %d",rc->sliders_att);
#endif
  emit toggleSplitSlidersSIGNAL();
  smenu->setItemChecked(smid[6], !rc->sliders_att); 
}

// =============================================================================== 

void winTVMainWidget::splitSlidersExtSLOT()
{
#ifdef mainDEBUG
  debug("main: splitSlidersExtSLOT %d",rc->sliders_att);
#endif
  smenu->setItemChecked(smid[6], !rc->sliders_att); 
}

// =============================================================================== 

void winTVMainWidget::soundSLOT()
{
  rc->sound_on=!rc->sound_on;

#ifdef mainDEBUG
  debug("main: soundSLOT: %s",rc->sound_on?"on":"off");
#endif

  
  if (! rc->use_tv_volume) 
    if (rc->sound_on) {
      mix->setVolume(rc->volume_left,VOL_LEFT);
      mix->setVolume(rc->volume_right,VOL_RIGHT);
    } else {
      mix->setVolume(0,VOL_BOTH);
    }
  emit audioMuteToggledSIGNAL();
  smenu->setItemChecked(smid[5], !rc->sound_on ); 
  QString number;
  number.setNum(rc->volume_left);number.append("%");
  statusbar->changeItem((const char*)number,SB_VOL);
}

// =============================================================================== 

void winTVMainWidget::volumeSLOT(int vl,int vr)
{
  if ((rc->volume_left != vl) || (rc->volume_right != vr)) {
#ifdef mainDEBUG
    debug("main: volumeSLOT %d, %d",vl,vr);
#endif
    rc->volume_left=vl;
    rc->volume_right=vr;

    if (!rc->use_tv_volume && rc->sound_on) {
      mix->setVolume(rc->volume_left,VOL_LEFT);
      mix->setVolume(rc->volume_right,VOL_RIGHT);
    }
    winTVScreen->showChannelInfoSLOT(NULL,rc->volume_right,0,MIXER_MAX_VALUE,"Volume");
    emit volumeChangedSIGNAL(rc->volume_left,rc->volume_right);
    QString number;
    number.setNum(rc->volume_left);number.append("%");
    statusbar->changeItem((const char*)number,SB_VOL);
  }
}

// =============================================================================== 

void winTVMainWidget::volumeUpSLOT()
{
  int vl=rc->volume_left+MIXER_STEP;
  int vr=rc->volume_right+MIXER_STEP;
  if (vl > MIXER_MAX_VALUE) vl=MIXER_MAX_VALUE;
  if (vr > MIXER_MAX_VALUE) vr=MIXER_MAX_VALUE;

  volumeSLOT(vl,vr);
}

// =============================================================================== 

void winTVMainWidget::volumeDownSLOT()
{
  int vl=rc->volume_left-MIXER_STEP;
  int vr=rc->volume_right-MIXER_STEP;
  if (vl < 0) vl=0;
  if (vr < 0) vr=0;

  volumeSLOT(vl,vr);
}

// =============================================================================== 

void winTVMainWidget::channelUpSLOT()
{
  channel_list.prev();
  if ( channel_list.at() == -1 ) channel_list.last();

#ifdef mainDEBUG
  debug("main: channel up: %s", channel_list.current()->name);
#endif

  emit channelChangedSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::channelDownSLOT()
{
  channel_list.next();
  if ( channel_list.at() == -1 ) channel_list.first();

#ifdef mainDEBUG
  debug("main: channel down: %s", channel_list.current()->name);
#endif
  emit channelChangedSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::newChannelsSLOT()
{
  // remove existing channels 
  if ( !channel_list.isEmpty() ) channel_list.clear();

  channel *s=new channel(rc->freq,"<no name>",rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in);
  channel_list.append( s );
  
  emit newChannelListSIGNAL( &channel_list );
  emit channelChangedSIGNAL();
}

// =============================================================================== 

void winTVMainWidget::loadChannels(const char *FILE,bool import)
{
  channel *n;

  ifstream fin(FILE);

  if (!fin) {
    warning(i18n("main: Can't load program-file %s!"),FILE);
    if ( channel_list.isEmpty() ) {
      channel *s=new channel(rc->freq,"<no name>",rc->col,rc->bri,rc->hue,rc->contr,rc->norm,rc->in);
      channel_list.append( s );
    }
    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
    return;
  }

  // remove existing channels 
  if ( !channel_list.isEmpty() ) channel_list.clear();

#ifdef mainDEBUG
    debug("main: loading channels from <%s>",FILE);
#endif

  while(fin.ignore(1000, '*') != NULL) {
    n= new channel();
    n->xtvscreen_file= import;

    fin >> *n;

    channel_list.append( n );

#ifdef mainDEBUG
      debug("main: load channel: %s, %d, %d",n->name, n->frequency, n->frequency*100/16);
#endif
  }
  
  if ( !channel_list.isEmpty() ) {
    if ( ! ((rc->ol_channel != -1 ) && channel_list.at(rc->ol_channel)) )
      channel_list.first();

    emit newChannelListSIGNAL( &channel_list );
    emit channelChangedSIGNAL();
#ifdef mainDEBUG
      debug("main: channel count: %d, at: %d",channel_list.count(),channel_list.at());
#endif
  }

}

// =============================================================================== 

void winTVMainWidget::saveChannels(const char *FILE)
{
  ofstream fout(FILE);
  if (!fout) {
    warning(i18n("main: Can't save program-file %s!"),FILE);
    return;
  }
  
  fout << "#" << endl;
  fout << "# kwintv programs file" << endl;
  fout << "#" << endl;

  channel *h= channel_list.first();

  while ( h ) { 
    fout << *h << endl;
    h= channel_list.next();
  }

  //chown(FILE,getuid(),getgid());

#ifdef mainDEBUG
  debug("main: channel-file <%s> saved!",FILE);
#endif
}

// =================================================================================================

void winTVMainWidget::closeEvent( QCloseEvent *e )
{
#ifdef mainDEBUG
  debug("main:closeEvent");
#endif
  kapp->quit();
}

// =================================================================================================

void winTVMainWidget::resizeEvent( QResizeEvent * theEvent)
{
  if (rc->autoar) adjustAspectRatioSLOT();
  
  if (isVisible()) kapp->sendEvent(winTVScreen,theEvent);

  updateRects();

#ifdef mainDEBUG
  debug("main: resize");
#endif
}

// =================================================================================================

void winTVMainWidget::moveEvent( QMoveEvent * theEvent)
{
  if (isVisible()) kapp->sendEvent(winTVScreen,theEvent);
#ifdef mainDEBUG
  debug("main: move");
#endif
}

// =================================================================================================

void winTVMainWidget::updateRects( )
{
  KTopLevelWidget::updateRects();

  if (rc->autoar) adjustAspectRatioSLOT();

#ifdef mainDEBUG
  debug("main: update");
#endif
}

// =================================================================================================

void winTVMainWidget::saveProperties (KConfig* config)
{
#ifdef mainDEBUG
  debug("main: save properties");
  debug("saving rc...");
#endif

  rc->save(); 
}

// =================================================================================================

void winTVMainWidget::readProperties (KConfig*)
{
#ifdef mainDEBUG
  debug("main: read properties");
#endif
}

//==============================================================================

void winTVMainWidget::switchToChannelSLOT( int dec )
{
#ifdef mainDEBUG
  debug("main: switch to channel %d, count: %d",dec,channel_list.count());
#endif
  if ( (channel_list.count() > (unsigned int)dec) && channel_list.at( dec ) ) {
    emit channelChangedSIGNAL();
    if (!capture) emit captureToggleSLOT();
  }
}

//==============================================================================

void winTVMainWidget::displaySpecialKeySLOT( KCodeEntry::codeType key)
{
#ifdef mainDEBUG
  debug("main: display special key");
#endif
  winTVScreen->showChannelInfoSLOT(NULL,CHINFO_NODISPVAL,
				   0, 511, "+10" );
}

// =================================================================================================

void winTVMainWidget::keyPressEvent ( QKeyEvent *keyE )
{
#ifdef mainDEBUG
  debug("main: key pressed 0x%x",keyE->key());
#endif
  if ( !keyup && (keyE->key() == KEY10UP) ) { keyup=true; key1=-1; key2=-1; }
  else if ( (keyE->key() >= Key_0) && (keyE->key() <= Key_9) ) {
    if ( keyup && (key1 == -1) ) {
      key1= keyE->key() - Key_0;
      return;
    } 
    key2= keyE->key() - Key_0;
    if ( keyup ) {
      key2+= key1 * 10;
      keyup=false;
    }
    switchToChannelSLOT( key2 );
  } else 
    keyE->ignore();
}

// =============================================================================== 

void winTVMainWidget::toggleTVonlySLOT()
{
#ifdef tvDEBUG
  debug("main: toggleTVonlySLOT");
#endif
  int Xoffset=0,Yoffset=0;
  QRect size=geometry();
  if (menubar->isVisible()) {
    // hide all menus
    Yoffset=menubar->height();
    menubar->hide();
    if (statusbar->isVisible()) { 
      Yoffset+=statusbar->height(); 
      statusbar->hide();
    }
    if (toolbar->isVisible()) { 
      if ((toolbar->barPos() == KToolBar::Left) || (toolbar->barPos() == KToolBar::Right))
	Xoffset+=toolbar->width();
      if((toolbar->barPos() == KToolBar::Top) || (toolbar->barPos() == KToolBar::Bottom))
	Yoffset+=toolbar->height();
      toolbar->hide();
    }
    resize(size.width()-Xoffset,size.height()-Yoffset);

    optmenu->setItemEnabled(omid[ID_HIDESTATUS],false);
    optmenu->setItemEnabled(omid[ID_HIDETOOL],false);
  } else {
    // show all menus
    menubar->show();
    if ( !optmenu->isItemChecked(omid[ID_HIDESTATUS]) ) statusbar->show();
    if ( !optmenu->isItemChecked(omid[ID_HIDETOOL]) ) toolbar->show();

    int Yoffset=menubar->height();
    if ( !optmenu->isItemChecked(omid[ID_HIDESTATUS]) ) Yoffset+=statusbar->height();
    if ( !optmenu->isItemChecked(omid[ID_HIDETOOL]) ) {
      if ((toolbar->barPos() == KToolBar::Left) || (toolbar->barPos() == KToolBar::Right))
	Xoffset+=toolbar->width();
      if((toolbar->barPos() == KToolBar::Top) || (toolbar->barPos() == KToolBar::Bottom))
	Yoffset+=toolbar->height();
    }
    resize(size.width()+Xoffset,size.height()+Yoffset);

    optmenu->setItemEnabled(omid[ID_HIDESTATUS],true);
    optmenu->setItemEnabled(omid[ID_HIDETOOL],true);
  }
  optmenu->setItemChecked(omid[ID_HIDEALL],!menubar->isVisible());
}

// =================================================================================================

void winTVMainWidget::emitSetupSignals()
{
#ifdef mainDEBUG
  debug("main: emit setup start");
#endif
  emit newChannelListSIGNAL( &channel_list );
  emit volumeChangedSIGNAL(rc->volume_left,rc->volume_right);

  if ( !rc->ol_statusbar ) toggleStatusBarSLOT();  
  if ( !rc->ol_toolbar ) toggleToolBarSLOT();
  if ( !rc->ol_menubar ) toggleMenuBarSLOT();

  if ( !rc->ol_statusbar || !rc->ol_menubar || !rc->ol_toolbar ) updateRects();

  if (!channel_list.isEmpty()) {
    emit channelChangedSIGNAL();
  }
  winTVScreen->postInit();
#ifdef mainDEBUG
  debug("main: emit setup");
#endif
}

// =================================================================================================

void winTVMainWidget::dataReceivedSLOT(int data)
{
#ifdef mainDEBUG
  debug("main: data received 0x%x",data);
#endif
}

// =================================================================================================

bool winTVMainWidget::x11Event( XEvent *event )
{
  Window wwin;

  if ((!isVisible())||(wwin=(Window) winId()) == None) return(false);

  switch ( event->type ) {
  case MapNotify:  
  case UnmapNotify:
#ifdef X11DEBUG
    debug("X11: winTVMainWidget::x11Event: got event %s",events[event->type]);
#endif
    emit x11EventSIGNAL( event );
    break;
  }

  return (false);
}

// =================================================================================================
// =================================================================================================

#include "wintvmain.moc"


